
# q-expansion of newform 9522.2.a.d, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9522
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [2], [0], [0], [-2], [2], [8], [0], [2], [-8], [-2], [-10], [-8], [-8], [2], [4], [-2], [-8], [0], [-6], [-8], [-16], [18], [-10], [18], [-8], [8], [-2], [-6], [16], [12], [10], [-20], [2], [16], [22], [-4], [0], [-14], [-12], [-10], [24], [-14], [-22], [-16], [-12], [24], [-8], [6], [22], [0], [22], [8], [-18], [8], [2], [-16], [-2], [2], [-24], [-30], [-4], [8], [-2], [-14], [-4], [30], [12], [6], [14], [0], [24], [-18], [16], [16], [-22], [30], [-14], [-22], [0], [30], [-24], [22], [-8], [-4], [30], [-18], [-30], [0], [0], [0], [-8], [-4], [-20], [-32], [-30], [26], [0], [-10], [-44], [2], [-24], [10], [-16], [18], [-12], [-34], [24], [42], [-24], [-10], [18], [40], [-24], [18], [-16], [-24], [10], [32], [-42], [18], [2], [4], [20], [18], [14], [-48], [0], [6], [-20], [-24], [-16], [38], [-42], [-26], [-38], [32], [2], [38], [-4], [50], [-32], [24], [22], [-56], [-26], [54], [36], [-32], [22], [-22], [4], [48], [-24], [40], [-16], [-34], [22], [58], [-44], [-22], [-32], [-48], [-14], [48], [-56], [14], [22], [-14], [8], [38], [24], [22], [0], [18], [4], [-46], [-56], [-10], [-56], [0], [-18], [6], [40], [-22], [30], [-32], [42], [40], [-46], [-28], [-32], [-6], [-48], [66], [30], [38], [42], [24], [18], [-16], [-34], [14], [16], [18], [40], [-4], [-26], [-12], [2], [-46], [-8], [48], [-32], [30], [-56], [30], [-24], [-54], [38], [-56], [46], [-64], [-12], [70], [50], [-24], [8], [28], [54], [48], [-64], [54], [-56], [-72], [6], [42], [20], [0], [24], [4], [-16], [-2], [14], [40], [16], [-48], [40], [-64], [-18], [-6], [-24], [-26], [42], [20], [-58], [-48], [-38], [42], [-32], [-48], [14], [78], [-18], [16], [-14], [-30], [-40], [42], [-74], [-48], [-58], [48], [-46], [24], [52], [78], [-10], [-24], [-48], [56], [32], [-58], [36], [-16], [38], [26], [40], [-2], [-70], [32], [-26], [48], [-26], [58], [-40], [-62], [76], [52], [-2], [-62], [24], [12], [32], [34], [-4], [62], [8], [6], [-32], [18], [-14], [4], [48], [-58], [-28], [88], [14], [30], [80], [46], [42], [8], [26], [-10], [64], [76], [0], [26], [-74], [26], [0], [-68], [-8], [60], [-58], [42], [-22], [-16], [-26], [-54], [-14], [-96], [66], [-12], [-74], [52], [-24], [82], [-4], [10], [-14], [40], [70], [54], [16], [72], [-2], [-96], [22], [-10], [-8], [-80], [20], [26], [90], [64], [-82], [4], [52], [-24], [-54], [-40], [14], [-84], [72], [30], [-46], [-6], [-14], [-86], [-24], [-82], [24], [-24], [-16], [14], [32], [24], [30], [90], [-76], [-60], [-48], [-2], [32], [-102], [-8], [-94], [14], [-2], [56], [-54], [-30], [32], [-82], [14], [32], [100], [-30], [-62], [40], [40], [22], [18], [8], [-64], [18], [48], [-94], [-34], [32], [-12], [58], [58], [-40], [6], [-20], [-80], [-2], [48], [-52], [-56], [-34], [66], [58], [54], [76], [88], [36], [42], [70], [96], [-30], [-2], [52], [0], [50], [46], [12], [0], [-108], [38], [-18], [74], [-74], [28], [6], [-94], [-60], [32], [-112], [-10], [4], [66]]
